@@ -131,6 +131,203 @@ class TestIplb(PrepareMockedIPLB, unittest.TestCase):
         self.mocked_client_instance.post.assert_called_with('/ipLoadbalancing/iplb_id/refresh')
 
 
+class TestFrontend(PrepareMockedIPLB, unittest.TestCase):
+    def setUp(self):
+        PrepareMockedIPLB.setUp(self)
+
+    def tearDown(self):
+        PrepareMockedIPLB.tearDown(self)
+
+    def test_exist_with_unexisting_id(self):
+        self.mock_get({
+            '/ipLoadbalancing/iplb_id/tcp/frontend/42': None,
+        })
+        self.assertFalse(self.iplb.frontend({
+            'type': 'tcp',
+            'id': 42
+        }).exist())
+
+    def test_exist_with_existing_id(self):
+        self.mock_get({
+            '/ipLoadbalancing/iplb_id/tcp/frontend/42': {'displayName': 'name'}
+        })
+        self.assertTrue(self.iplb.frontend({
+            'type': 'tcp',
+            'id': 42,
+            'default_farm_name': 'farm_name'
+        }).exist())
+
+    def test_exist_with_unexisting_name(self):
+        self.mock_get({
+            '/ipLoadbalancing/iplb_id/tcp/frontend/42': {'frontendId': 42, 'displayName': 'name'},
+            '/ipLoadbalancing/iplb_id/tcp/frontend': ['42']
+        })
+        self.assertFalse(self.iplb.frontend({
+            'type': 'tcp',
+            'name': 'bad_name'
+        }).exist())
+
+    def test_exist_with_existing_name(self):
+        self.mock_get({
+            '/ipLoadbalancing/iplb_id/tcp/farm/42': {'farmId': 42, 'displayName': 'name'},
+            '/ipLoadbalancing/iplb_id/tcp/farm': ['42']
+        })
+        self.assertTrue(self.iplb.farm({
+            'type': 'tcp',
+            'name': 'name'
+        }).exist())
+
+    def test_exist_with_duplicated_name(self):
+        self.mock_get({
+            '/ipLoadbalancing/iplb_id/tcp/frontend/42': {'frontenId': 42, 'displayName': 'name'},
+            '/ipLoadbalancing/iplb_id/tcp/frontend/41': {'frontenId': 41, 'displayName': 'name'},
+            '/ipLoadbalancing/iplb_id/tcp/frontend': ['41', '42']
+        })
+        with self.assertRaises(ValueError):
+            self.iplb.frontend({
+                'type': 'tcp',
+                'name': 'name',
+                'default_farm_name': 'name'
+
+            }).exist()
+
+    def test_create_should_fail_when_no_farm_with_given_name_exist(self):
+        self.mock_get({
+            '/ipLoadbalancing/iplb_id/tcp/farm': ['42'],
+            '/ipLoadbalancing/iplb_id/tcp/farm/42': {'farmId': 42, 'displayName': 'name'}
+        })
+        with self.assertRaises(ValueError):
+            self.iplb.frontend({
+                'id': '123456789',
+                'type': 'tcp',
+                'allowed_source': '1.2.3.4',
+                'dedicated_ipfo': '6.6.6.6',
+                'default_farm_name': 'bad_name',
+                'default_ssl_id': 666,
+                'disabled': False,
+                'name': 'name',
+                'http_header': 'X-super-header',
+                'hsts': False,
+                'port': '666',
+                'redirect_location': 'to_bikkini_bottom',
+                'ssl': False,
+                'zone': 'all'
+             }).body()
+
+    def test_body_with_default_farm_id(self):
+        self.assertEqual(self.iplb.frontend({
+            'id': '123456789',
+            'type': 'tcp',
+            'allowed_source': '1.2.3.4',
+            'dedicated_ipfo': '6.6.6.6',
+            'default_farm_id': 1234,
+            'default_ssl_id': 666,
+            'disabled': False,
+            'name': 'name',
+            'http_header': 'X-super-header',
+            'hsts': False,
+            'port': '666',
+            'redirect_location': 'to_bikkini_bottom',
+            'ssl': False,
+            'zone': 'all'
+        }).body(), {
+            'displayName': 'name',
+            'port': '666',
+            'zone': 'all',
+            'allowedSource': '1.2.3.4',
+            'dedicatedIpfo': '6.6.6.6',
+            'defaultFarmId': 1234,
+            'defaultSslId': 666,
+            'disabled': False,
+            'hsts': False,
+            'httpHeader': 'X-super-header',
+            'redirectLocation': 'to_bikkini_bottom',
+            'ssl': False
+        })
+
+    def test_body_when_default_farm_name(self):
+        self.mock_get({
+            '/ipLoadbalancing/iplb_id/tcp/farm': ['42'],
+            '/ipLoadbalancing/iplb_id/tcp/farm/42': {'farmId': 42, 'displayName': 'name'}
+        })
+        with self.assertRaises(ValueError):
+            self.assertEqual(
+                self.iplb.frontend({
+                    'id': '123456789',
+                    'type': 'tcp',
+                    'allowed_source': '1.2.3.4',
+                    'dedicated_ipfo': '6.6.6.6',
+                    'default_farm_name': 'name',
+                    'default_name_id': None,
+                    'default_ssl_id': 666,
+                    'disabled': False,
+                    'name': 'name',
+                    'http_header': 'X-super-header',
+                    'hsts': False,
+                    'port': '666',
+                    'redirect_location': 'to_bikkini_bottom',
+                    'ssl': False,
+                    'zone': 'all'
+                }).body(),
+                {
+                    'displayName': 'name',
+                    'port': '666',
+                    'zone': 'all',
+                    'allowedSource': '1.2.3.4',
+                    'dedicatedIpfo': '6.6.6.6',
+                    'defaultFarmId': 42,
+                    'defaultSslId': 666,
+                    'disabled': False,
+                    'hsts': False,
+                    'httpHeader': 'X-super-header',
+                    'redirectLocation': 'to_bikkini_bottom',
+                    'ssl': False
+                }
+            )
+
+    def test_body_with_both_default_farm_name_id(self):
+        self.mock_get({
+            '/ipLoadbalancing/iplb_id/tcp/farm': ['42'],
+            '/ipLoadbalancing/iplb_id/tcp/farm/42': {'farmId': 42, 'displayName': 'name'}
+        })
+        with self.assertRaises(ValueError):
+            self.iplb.frontend({
+                'id': '123456789',
+                'type': 'tcp',
+                'allowed_source': '1.2.3.4',
+                'dedicated_ipfo': '6.6.6.6',
+                'default_farm_id': '42',
+                'default_farm_name': 'name',
+                'default_ssl_id': 666,
+                'disabled': False,
+                'name': 'name',
+                'http_header': 'X-super-header',
+                'hsts': False,
+                'port': '666',
+                'redirect_location': 'to_bikkini_bottom',
+                'ssl': False,
+                'zone': 'all'
+                }).body()
+
+    def test_read(self):
+        self.mock_get({
+            '/ipLoadbalancing/iplb_id/tcp/frontend/42': {'frontendId': 42, 'displayName': 'name', 'field': 'data'},
+            '/ipLoadbalancing/iplb_id/tcp/frontend/41': {'frontendId': 41, 'displayName': 'otherName'},
+            '/ipLoadbalancing/iplb_id/tcp/frontend': ['41', '42']
+        })
+
+        self.assertEqual(self.iplb.frontend({'type': 'tcp', 'name': 'name'}).read(), {'frontendId': 42,
+                                                                                      'displayName': 'name',
+                                                                                      'field': 'data'})
+
+    def test_create(self):
+        self.iplb.frontend({'default_farm_id': 42, 'type': 'tcp', 'name': 'name', 'zone': 'gra'}).create()
+        self.mocked_client_instance.post.assert_called_with('/ipLoadbalancing/iplb_id/tcp/frontend',
+                                                            displayName='name',
+                                                            defaultFarmId=42,
+                                                            zone='gra')
+
+
 class TestFarm(PrepareMockedIPLB, unittest.TestCase):
     def setUp(self):
         PrepareMockedIPLB.setUp(self)
